package net.hostsharing.hsadminng.hs.booking.item;

import lombok.Getter;
import org.springframework.context.ApplicationEvent;

import jakarta.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Getter
public class BookingItemCreatedEvent extends ApplicationEvent {

    private static final Map<UUID, BookingItemCreatedEvent> events = new HashMap<>(); // FIXME: use DB table

    static BookingItemCreatedEvent of(final HsBookingItemRealEntity bookingItem) {
        return events.get(bookingItem.getUuid());
    }

    @Getter
    public static class Status {

        private final String message;

        private Status(final String message) {
            this.message = message;
        }

        public static Status finished() {
            return new Status(null);
        }

        public static Status failed(final String errorMessage) {
            return new Status(errorMessage);
        }

        boolean isFinished() {
            return message == null;
        }
    }

    private final @NotNull UUID bookingItemUuid;
    private final @NotNull String assetJson;

    private Status status;

    public BookingItemCreatedEvent(
            @NotNull final HsBookingItemController source,
            @NotNull final HsBookingItem newBookingItem,
            final String assetJson) {
        super(source);
        this.bookingItemUuid = newBookingItem.getUuid();
        this.assetJson = assetJson;
    }

    public void setStatus(final Status status) {
        this.status = status;
        events.put(bookingItemUuid, this);
    }
}
